from damask import __version__
from damask_parse.writers import write_load_case


def write_load(path, load_case):

    if not isinstance(load_case, list):
        load_case = [load_case]

    load_cases = [i.to_dict() for i in load_case]
    write_load_case(
        dir_path=path.parent,
        load_cases=load_cases,
        name=path.name,
        write_2D_arrs=(__version__ != "3.0.0-alpha3"),
    )



import copy
import numpy as np
from damask import Grid
from damask_parse.utils import validate_volume_element, validate_orientations


def generate_volume_element_random_voronoi(
    microstructure_seeds,
    VE_grid_size,
    homog_label,
):
    grid_obj = Grid.from_Voronoi_tessellation(
        cells=np.array(VE_grid_size),
        size=np.array(microstructure_seeds["size"]),
        seeds=np.array(microstructure_seeds["position"]),
        periodic=True,
    )
    oris = copy.deepcopy(microstructure_seeds["orientations"])
    oris = validate_orientations(oris)

    num_grains = len(microstructure_seeds["position"])
    const_phase_lab = np.array(microstructure_seeds["phase_labels"])[
        np.zeros(num_grains, dtype=int)
    ]

    ori_idx = np.arange(num_grains)

    volume_element = {
        "size": grid_obj.size.astype(float).tolist(),
        "grid_size": grid_obj.cells.tolist(),
        "orientations": oris,
        "element_material_idx": grid_obj.material,
        "constituent_material_idx": np.arange(num_grains),
        "constituent_material_fraction": np.ones(num_grains),
        "constituent_phase_label": const_phase_lab,
        "constituent_orientation_idx": ori_idx,
        "material_homog": np.full(num_grains, homog_label),
    }
    volume_element = validate_volume_element(volume_element)
    return {"volume_element": volume_element}



from dataclasses import dataclass
import enum
from typing import Optional

import numpy as np
from hpcflow.sdk.core.parameters import ParameterValue


@dataclass
class OrientationFormat(enum.Enum):

    # i.e. InputValue(parameter=Parameter(typ='orientations'), path=('ori_format',), value=...)

    QUATERNION = 1
    EULER = 2


@dataclass
class Orientations(ParameterValue):

    # i.e. InputValue(parameter=Parameter(typ='orientations'), value=...)
    _typ = "orientations"

    # _sub_parameters = (
    #     SubParameter(path=("ori_format",), is_enum=True, class_obj=OrientationFormat),
    # )

    data: np.ndarray
    ori_format: Optional[OrientationFormat] = None

    def __post_init__(self):
        pass
        # if not isinstance(self.ori_format, OrientationFormat):
        #     self.ori_format = getattr(OrientationFormat, self.ori_format.upper())

    @classmethod
    def from_JSON_like(cls, data, ori_format):
        """For custom initialisation via YAML or JSON."""
        return cls(data=np.asarray(data), ori_format=ori_format)

    # def to_json_like(self, dct=None, shared_data=None, exclude=None, _depth=0):
    #     return

    @classmethod
    def from_random(cls, number):
        return cls(
            data=cls.quat_sample_random(number),
            ori_format="quaternion",
        )

    @staticmethod
    def quat_sample_random(number):
        """Generate random uniformly distributed unit quaternions.
        Parameters
        ----------
        number : int
            How many quaternions to generate.
        Returns
        -------
        quats : ndarray of shape (number, 4)
        References
        ----------
        https://stackoverflow.com/a/44031492/5042280
        http://planning.cs.uiuc.edu/node198.html
        """

        rand_nums = np.random.random((number, 3))
        quats = np.array(
            [
                np.sqrt(1 - rand_nums[:, 0]) * np.sin(2 * np.pi * rand_nums[:, 1]),
                np.sqrt(1 - rand_nums[:, 0]) * np.cos(2 * np.pi * rand_nums[:, 1]),
                np.sqrt(rand_nums[:, 0]) * np.sin(2 * np.pi * rand_nums[:, 2]),
                np.sqrt(rand_nums[:, 0]) * np.cos(2 * np.pi * rand_nums[:, 2]),
            ]
        ).T

        return quats
